@@ -12,7 +12,7 @@ from string import Template
 
 from nereid import (
     render_template, current_app, cache, request, login_required, jsonify,
-    redirect, flash, abort
+    redirect, flash, abort, route
 )
 from nereid.helpers import slugify, url_for, key_from_list
 from nereid.contrib.pagination import Pagination
@@ -541,7 +541,10 @@ class ArticleCategory(ModelSQL, ModelView):
     active = fields.Boolean('Active', select=True)
     description = fields.Text('Description', translate=True)
     template = fields.Char('Template', required=True)
-    articles = fields.One2Many('nereid.cms.article', 'category', 'Articles')
+    articles = fields.One2Many(
+        'nereid.cms.article', 'category', 'Articles',
+        context={'published': True}
+    )
 
     # Article Category can have a banner
     banner = fields.Many2One('nereid.cms.banner', 'Banner')
@@ -549,6 +552,11 @@ class ArticleCategory(ModelSQL, ModelView):
         ('older_first', 'Older First'),
         ('recent_first', 'Recent First'),
     ], 'Sort Order')
+    published_articles = fields.Function(
+        fields.One2Many(
+            'nereid.cms.article', 'category', 'Published Articles'
+        ), 'get_published_articles'
+    )
 
     @staticmethod
     def default_sort_order():
@@ -578,6 +586,8 @@ class ArticleCategory(ModelSQL, ModelView):
         return res
 
     @classmethod
+    @route('/article-category/<uri>/')
+    @route('/article-category/<uri>/<int:page>')
     def render(cls, uri, page=1):
         """
         Renders the category
@@ -624,11 +634,13 @@ class ArticleCategory(ModelSQL, ModelView):
         return {'get_article_category': cls.get_article_category}
 
     @classmethod
+    @route('/sitemaps/article-category-index.xml')
     def sitemap_index(cls):
         index = SitemapIndex(cls, [])
         return index.render()
 
     @classmethod
+    @route('/sitemaps/article-category-<int:page>.xml')
     def sitemap(cls, page):
         sitemap_section = SitemapSection(cls, [], page)
         sitemap_section.changefreq = 'daily'
@@ -640,8 +652,20 @@ class ArticleCategory(ModelSQL, ModelView):
             uri=self.unique_name, **kwargs
         )
 
+    def get_published_articles(self, name):
+        """
+        Get the published articles.
+        """
+        NereidArticle = Pool().get('nereid.cms.article')
 
-class Article(ModelSQL, ModelView):
+        articles = NereidArticle.search([
+            ('state', '=', 'published'),
+            ('category', '=', self.id)
+        ])
+        return map(int, articles)
+
+
+class Article(Workflow, ModelSQL, ModelView):
     "CMS Articles"
     __name__ = 'nereid.cms.article'
     _rec_name = 'uri'
@@ -671,6 +695,11 @@ class Article(ModelSQL, ModelView):
 
     # Article can have a banner
     banner = fields.Many2One('nereid.cms.banner', 'Banner')
+    state = fields.Selection([
+        ('draft', 'Draft'),
+        ('published', 'Published'),
+        ('archived', 'Archived')
+    ], 'State', required=True, select=True, readonly=True)
 
     @classmethod
     def __register__(cls, module_name):
@@ -688,6 +717,31 @@ class Article(ModelSQL, ModelView):
     def __setup__(cls):
         super(Article, cls).__setup__()
         cls._order.insert(0, ('sequence', 'ASC'))
+        cls._transitions |= set((
+                ('draft', 'published'),
+                ('archived', 'draft'),
+                ('published', 'archived'),
+        ))
+        cls._buttons.update({
+            'archive': {
+                'invisible': Eval('state') != 'published',
+            },
+            'publish': {
+                'invisible': Eval('state') == 'published',
+            }
+        })
+
+    @classmethod
+    @ModelView.button
+    @Workflow.transition('archived')
+    def archive(cls, articles):
+        pass
+
+    @classmethod
+    @ModelView.button
+    @Workflow.transition('published')
+    def publish(cls, articles):
+        pass
 
     @staticmethod
     def links_get():
@@ -733,6 +787,7 @@ class Article(ModelSQL, ModelView):
         return Date.today()
 
     @classmethod
+    @route('/article/<uri>')
     def render(cls, uri):
         """
         Renders the template
@@ -744,11 +799,13 @@ class Article(ModelSQL, ModelView):
         return render_template(article.template, article=article)
 
     @classmethod
+    @route('/sitemaps/article-index.xml')
     def sitemap_index(cls):
         index = SitemapIndex(cls, [])
         return index.render()
 
     @classmethod
+    @route('/sitemaps/article-<int:page>.xml')
     def sitemap(cls, page):
         sitemap_section = SitemapSection(cls, [], page)
         sitemap_section.changefreq = 'daily'
@@ -769,6 +826,12 @@ class Article(ModelSQL, ModelView):
             'nereid.cms.article.render', uri=self.uri, **kwargs
         )
 
+    @staticmethod
+    def default_state():
+        if 'published' in Transaction().context:
+            return 'published'
+        return 'draft'
+
 
 class ArticleAttribute(ModelSQL, ModelView):
     "Articles Attribute"
@@ -818,6 +881,7 @@ class Website:
     )
 
     @classmethod
+    @route('/cms/upload/<upload_type>', methods=['POST'])
     @login_required
     def cms_static_upload(cls, upload_type):
         """
@@ -845,6 +909,8 @@ class Website:
         return redirect(request.referrer)
 
     @classmethod
+    @route('/cms/browse', methods=['POST'])
+    @route('/cms/browse/<int:page>', methods=['GET'])
     @login_required
     def cms_static_list(cls, page=1):
         """
